/*
** Module   : XDocumentImpl
** Abstract : DOM XML Document resource type implementation.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130204 Created initial version. Major rework to use w3c backend.
** 002 OM  20130304 Refactored isValid and isUnknown of WrappedResource to valid and unknown.
** 003 CA  20130308 Refactored as XmlFactory APIs now receive a handle instance instead of 
**                  returning a new one.
*/

package com.goldencode.p2j.xml;

import java.io.*;
import org.w3c.dom.*;
import com.goldencode.p2j.util.*;
import com.goldencode.util.XmlHelper;

/**
 * Implementation of the X-Document specific features of the Progress DOM XML
 * support objects.
 */
public class XDocumentImpl
extends XEntityImpl
implements XDocument
{
   /** The real document to assign with handle.  */
   private Document xdoc = null;
   
   /** The root node of the current X-Document. */
   private XNodeRefImpl xRootNode = null;

   /**
    * Default constructor.
    */
   public XDocumentImpl()
   {
      super();
      try
      {
         xdoc = XmlHelper.newDocument();
      }
      catch (Exception e)
      {
         // TODO: change this
         xdoc = null;
      }
      xRootNode = new XNodeRefImpl();
   }

   // Methods
   // Returning value placed into hChild
   /**
    * Creating the new node within the current Document object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNode(handle hChild, String name, String type)
   {
      return createNode(hChild, new character(name), new character(type));
   }

   /**
    * Creating the new node within the current Document object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNode(handle hChild, character name, String type)
   {
      return createNode(hChild, name, new character(type));
   }

   /**
    * Creating the new node within the current Document object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNode(handle hChild, String name, character type)
   {
      return createNode(hChild, new character(name), type);
   }

   /**
    * Creating the new node within the current Document object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNode(handle hChild, character name, character type)
   {
      XmlFactory.createXNodeReference(hChild, xdoc, name, type);
      
      return hChild.isValid();
   }

   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, String uri,
                                      String name, String type)
   {
      return createNodeNamespace(hChild, new character(uri),
                                 new character(name), new character(type));
   }
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, character uri,
                                      String name, String type)
   {
      return createNodeNamespace(hChild, uri, new character(name),
                                 new character(type));
   }
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, String uri,
                                      character name, String type)
   {
      return createNodeNamespace(hChild, new character(uri), name,
                                 new character(type));
   }
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, String uri,
                                      String name, character type)
   {
      return createNodeNamespace(hChild, new character(uri),
                                 new character(name), type);
   }
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, String uri,
                                      character name, character type)
   {
      return createNodeNamespace(hChild, new character(uri), name, type);
   }
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, character uri,
                                      String name, character type)
   {
      return createNodeNamespace(hChild, uri, new character(name), type);
   }
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, character uri,
                                      character name, String type)
   {
      return createNodeNamespace(hChild, uri, name, new character(type));
   }
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, character uri,
                                      character name, character type)
   {
      XmlFactory.createXNodeReference(hChild, xdoc, uri, name, type);
      
      return hChild.isValid();
   }
   
   // Returning value placed into hRoot
   /**
    * Getting the root node for the current document. The handle is placed
    * into passed object reference.
    * 
    * @param    hRoot
    *           The valid handle class instance variable to receive the
    *           result.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical getDocumentElement(handle hRoot)
   {
      hRoot = getHandle();
      if (valid())
      {
         return new logical(true);
      }
      else
      {
         return new logical(false);
      }
   }
   
   /**
    * Loading X-Document form file or memptr sources.
    * 
    * @param    mode
    *           Variable indicating the type of source to read. "file" means
    *           reading from file, "memptr" means reading from memory buffer.
    * @param    source
    *           If mode == "file" this is the file name, if "memptr" this is
    *           the memptr object.
    * @param    validate
    *           Flag indicating whether to validate while reading or not.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical load(String mode, Object source, logical validate)
   {
      return load(new character(mode), source, validate);
   }
   
   /**
    * Loading X-Document form file or memptr sources.
    * 
    * @param    mode
    *           Variable indicating the type of source to read. "file" means
    *           reading from file, "memptr" means reading from memory buffer.
    * @param    source
    *           If mode == "file" this is the file name, if "memptr" this is
    *           the memptr object.
    * @param    validate
    *           Flag indicating whether to validate while reading or not.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical load(String mode, Object source, boolean validate)
   {
      return load(new character(mode), source, new logical(validate));
   }
   
   /**
    * Loading X-Document form file or memptr sources.
    * 
    * @param    mode
    *           Variable indicating the type of source to read. "file" means
    *           reading from file, "memptr" means reading from memory buffer.
    * @param    source
    *           If mode == "file" this is the file name, if "memptr" this is
    *           the memptr object.
    * @param    validate
    *           Flag indicating whether to validate while reading or not.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical load(character mode, Object source, boolean validate)
   {
      return load(mode, source, new logical(validate));
   }
   
   /**
    * Loading X-Document form file or memptr sources.
    * 
    * @param    mode
    *           Variable indicating the type of source to read. "file" means
    *           reading from file, "memptr" means reading from memory buffer.
    * @param    source
    *           If mode == "file" this is the file name, if "memptr" this is
    *           the memptr object.
    * @param    validate
    *           Flag indicating whether to validate while reading or not.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical load(character mode, Object source, logical validate)
   {
      String loadMode = mode.getValue();
      
      if (loadMode.equalsIgnoreCase("file"))
      {
         try
         {
            xdoc = XmlHelper.parse((String)source, validate.getValue(), null);
         }
         catch (Exception e)
         {
//            throw new IOException("Error reading XML document from file", e);
         }
      }
      else if (loadMode.equalsIgnoreCase("memptr"))
      {
         return new logical(false);
      }
      else
      {
         return new logical(false);
      }
      
      return new logical(true);
   }
   
   /**
    * Saving X-Document to file or stream or memptr object.
    * 
    * @param    mode
    *           Variable indicating the type of target to write. "file" means
    *           writing to file, "stream" means writing to the stream and
    *           "memptr" means writing to memory buffer.
    * @param    target
    *           If mode == "file" this is the file name, if "stream" means
    *           output stream, if "memptr" this is the memptr object.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical save(String mode, Object target)
   {
      return save(new character(mode), target);
   }
   
   /**
    * Saving X-Document to file or stream or memptr object.
    * 
    * @param    mode
    *           Variable indicating the type of target to write. "file" means
    *           writing to file, "stream" means writing to the stream and
    *           "memptr" means writing to memory buffer.
    * @param    target
    *           If mode == "file" this is the file name, if "stream" means
    *           output stream, if "memptr" this is the memptr object.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical save(character mode, Object target)
   {
      String writeMode = mode.getValue();
      if (writeMode.equalsIgnoreCase("file"))
      {
         try
         {
            XmlHelper.write(xdoc, (String)target);
         }
         catch (Exception e)
         {
//            throw new IOException("Error writing XML document to file", e);
         }
      }
      else if (writeMode.equalsIgnoreCase("stream"))
      {
         try
         {
            XmlHelper.write(xdoc, (OutputStream)target);
         }
         catch (Exception e)
         {
//            throw new IOException("Error writing XML document to stream", e);
         }
      }
      else if (writeMode.equalsIgnoreCase("memptr"))
      {
         return new logical(false);
      }
      else
      {
         return new logical(false);
      }
      
      return new logical(true);
   }
}
